/*
 *  -- MAGMA (version 1.1) --
 *     Univ. of Tennessee, Knoxville
 *     Univ. of California, Berkeley
 *     Univ. of Colorado, Denver
 *     April 2012
 *
 * @generated c Fri Nov 22 14:21:32 2013
 *
 **/
// includes, system
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

// includes, project
#include "flops.h"
#include "magma.h"
#include "magma_lapack.h"
#include "magma_types.h"
#include "testings.h"

/* ////////////////////////////////////////////////////////////////////////////
   -- Testing cpotrf
*/
int main( int argc, char** argv)
{
	/* Initialize */
	magma_queue_t  queue;
	magma_device_t device;
	magma_int_t num = 0;
	magma_err_t err;

	magma_init();
	err = magma_get_devices( &device, 1, &num );
	if ( err != MAGMA_SUCCESS or num < 1 ) {
		fprintf( stderr, "magma_get_devices failed: %d\n", err );
		exit(-1);
	}
	err = magma_queue_create( device, &queue );
	if ( err != MAGMA_SUCCESS ) {
		fprintf( stderr, "magma_queue_create failed: %d\n", err );
		exit(-1);
	}

	real_Double_t   gflops, mic_perf, mic_time, cpu_perf, cpu_time;
	magmaFloatComplex *h_A, *h_R;
	magmaFloatComplex *d_A;
	magma_int_t N, n2, lda, ldda, info;
	magmaFloatComplex c_neg_one = MAGMA_C_NEG_ONE;
	magma_int_t ione     = 1;
	magma_int_t ISEED[4] = {0,0,0,1};
	float      work[1], error;

	magma_opts opts;
	parse_opts( argc, argv, &opts );

	printf("  N     CPU GFlop/s (sec)   MAGMA GFlop/s (sec)   ||R_magma - R_lapack||_F / ||R_lapack||_F\n");
	printf("=============================================================================================\n");
	for( int i = 0; i < opts.ntest; ++i ) {
		for( int iter = 0; iter < opts.niter; ++iter ) {
			N   = opts.nsize[i];
			lda = N;
			n2  = lda*N;
			ldda = ((N+31)/32)*32;
			gflops = FLOPS_CPOTRF( N ) / 1e9;

			TESTING_MALLOC_HOST(    h_A, magmaFloatComplex, n2     );
			TESTING_MALLOC_HOST( h_R, magmaFloatComplex, n2     );
			TESTING_MALLOC_DEV(  d_A, magmaFloatComplex, ldda*N );

			/* Initialize the matrix */
			lapackf77_clarnv( &ione, ISEED, &n2, h_A );
			magma_cmake_hpd( N, h_A, lda );
			lapackf77_clacpy( MagmaUpperLowerStr, &N, &N, h_A, &lda, h_R, &lda );
			magma_csetmatrix( N, N, h_A, 0, lda, d_A, 0, ldda, queue );
			magma_cpotrf_mic( opts.uplo, N, d_A, 0, ldda, &info, queue );

			magma_csetmatrix( N, N, h_A, 0, lda, d_A, 0, ldda, queue );
			/* ====================================================================
			   Performs operation using MAGMA
			   =================================================================== */
			mic_time = magma_wtime();
			magma_cpotrf_mic( opts.uplo, N, d_A, 0, ldda, &info, queue );
			mic_time = magma_wtime() - mic_time;
			mic_perf = gflops / mic_time;
			if (info != 0)
				printf("magma_cpotrf_mic returned error %d.\n", (int) info);

			if ( opts.check ) {
				/* =====================================================================
				   Performs operation using LAPACK
				   =================================================================== */
				cpu_time = magma_wtime();
				lapackf77_cpotrf( lapack_const(opts.uplo), &N, h_A, &lda, &info );
				cpu_time = magma_wtime() - cpu_time;
				cpu_perf = gflops / cpu_time;
				if (info != 0)
					printf("lapackf77_cpotrf returned error %d.\n", (int) info);

				/* =====================================================================
				   Check the result compared to LAPACK
				   =================================================================== */
				magma_cgetmatrix( N, N, d_A, 0, ldda, h_R, 0, lda, queue );
				//error = lapackf77_clange("f", &N, &N, h_A, &lda, work);
				error = lapackf77_clanhe("f", "L", &N, h_A, &lda, work);
				blasf77_caxpy(&n2, &c_neg_one, h_A, &ione, h_R, &ione);
				//error = lapackf77_clange("f", &N, &N, h_R, &lda, work) / error;
				error = lapackf77_clanhe("f", "L", &N, h_R, &lda, work) / error;

				printf("%5d   %7.2f (%7.2f)   %7.2f (%7.2f)   %8.2e\n",
						(int) N, cpu_perf, cpu_time, mic_perf, mic_time, error );
			}
			else {
				printf("%5d     ---   (  ---  )   %7.2f (%7.2f)     ---  \n",
						(int) N, mic_perf, mic_time );
			}

			fflush(0);
			TESTING_FREE_HOST(     h_A );
			TESTING_FREE_HOST( h_R );
			TESTING_FREE_DEV(  d_A );
		}
		if ( opts.niter > 1 ) {
			printf( "\n" );
		}
	}

	/* Shutdown */
	magma_queue_destroy( queue );
    magma_finalize();

    return 0;
}
