/*
 *  -- MAGMA (version 1.1) --
 *     Univ. of Tennessee, Knoxville
 *     Univ. of California, Berkeley
 *     Univ. of Colorado, Denver
 *     April 2012
 *
 * @generated c Fri Nov 22 14:21:30 2013
 *
 **/

// make sure that asserts are enabled
#ifdef NDEBUG
#undef NDEBUG
#endif
#include <assert.h>

// includes, system
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

// includes, project
#include "flops.h"
#include "magma.h"
#include "magma_lapack.h"
#include "magma_types.h"
#include "testings.h"

/* ////////////////////////////////////////////////////////////////////////////
   -- Testing ctrsm
*/
int main( int argc, char** argv)
{
    /* Initialize */
    magma_queue_t  queue;
    magma_device_t device;
    int num = 0;
    magma_err_t err;
    magma_init();
    err = magma_get_devices( &device, 1, &num );
    if ( err != MAGMA_SUCCESS or num < 1 ) {
      fprintf( stderr, "magma_get_devices failed: %d\n", err );
      exit(-1);
    }
    err = magma_queue_create( device, &queue );
    if ( err != MAGMA_SUCCESS ) {
      fprintf( stderr, "magma_queue_create failed: %d\n", err );
      exit(-1);
    }

    real_Double_t   gflops, mic_perf, mic_time, cpu_perf, cpu_time;
    float          mic_error, Cnorm, work[1];
    magma_int_t N, info;
    magma_int_t Ak;
    magma_int_t sizeA;
    magma_int_t lda, ldda;
    magma_int_t ione     = 1;
    magma_int_t ISEED[4] = {0,0,0,1};
   
    magma_int_t *piv;

    magmaFloatComplex *h_A, *h_x, *h_b1, *h_xmic, *h_x1, *LU, *LUT;
    magmaFloatComplex *d_A, *d_x;
    magmaFloatComplex c_neg_one = MAGMA_C_NEG_ONE;
    magmaFloatComplex c_one = MAGMA_C_ONE;
    
    magma_opts opts;
    parse_opts( argc, argv, &opts );
    
    printf("    N   MIC Gflop/s (ms)   CPU Gflop/s (ms)  MIC error\n");
    printf("==========================================================\n");
    for( int i = 0; i < opts.ntest; ++i ) {
        for( int iter = 0; iter < opts.niter; ++iter ) {
            N = opts.nsize[i];
            gflops = FLOPS_CTRSM(opts.side, N, 1) / 1e9;

            lda = N;
            Ak = N;
            
            ldda = ((lda+31)/32)*32;
            
            sizeA = lda*Ak;
            
            TESTING_MALLOC_HOST( h_A,  magmaFloatComplex, lda*Ak );
            TESTING_MALLOC_HOST( LU,      magmaFloatComplex, lda*Ak );
            TESTING_MALLOC_HOST( LUT,  magmaFloatComplex, lda*Ak );
            TESTING_MALLOC_HOST( h_x,  magmaFloatComplex, N  );
            TESTING_MALLOC_HOST( h_x1,  magmaFloatComplex, N );
            TESTING_MALLOC_HOST( h_b1,  magmaFloatComplex, N );
            TESTING_MALLOC_HOST( h_xmic, magmaFloatComplex, N  );
            
            TESTING_MALLOC_DEV( d_A, magmaFloatComplex, ldda*Ak );
            TESTING_MALLOC_DEV( d_x, magmaFloatComplex, N  );
            
            /* Initialize the matrices */
            lapackf77_clarnv( &ione, ISEED, &sizeA, LU );
            TESTING_MALLOC_HOST(piv, magma_int_t, N );  assert( err == 0 );
            lapackf77_cgetrf( &Ak, &Ak, LU, &lda, piv, &info );
            
            int i, j;
            for(i=0;i<Ak;i++){
                for(j=0;j<Ak;j++){
                    LUT[j+i*lda] = LU[i+j*lda];
                }
            }

            lapackf77_clacpy(MagmaUpperStr, &Ak, &Ak, LUT, &lda, LU, &lda);
            
            if(opts.uplo == MagmaLower){
                lapackf77_clacpy(MagmaLowerStr, &Ak, &Ak, LU, &lda, h_A, &lda);
            }else{
                lapackf77_clacpy(MagmaUpperStr, &Ak, &Ak, LU, &lda, h_A, &lda);
            }
            
            lapackf77_clarnv( &ione, ISEED, &N, h_x );
            memcpy(h_b1, h_x, N*sizeof(magmaFloatComplex));
            /* =====================================================================
               Performs operation using CUDA-BLAS
               =================================================================== */
            magma_csetmatrix( Ak, Ak, h_A, 0, lda, d_A, 0, ldda, queue );
            //magma_csetvector( N, h_x, 1, d_x, 1 );
            magma_csetmatrix( N, 1, h_x, 0, N, d_x, 0, N, queue );
            
            mic_time = magma_sync_wtime( NULL );
            magma_ctrsv( opts.uplo, opts.transA, opts.diag,
                         N, 
                         d_A, 0, ldda,
                         d_x, 0, 1, queue );
            mic_time = magma_sync_wtime( NULL ) - mic_time;
            mic_perf = gflops / mic_time;
            
	    /* =====================================================================
	       Performs operation using CPU BLAS
	       =================================================================== */
	    if ( opts.check ) {
		    //magma_cgetvector( N, d_x, 1, h_xmic, 1 );
		    magma_cgetmatrix( N, 1, d_x, 0, N, h_xmic, 0, N, queue );
		    cpu_time = magma_wtime();
		    blasf77_ctrsv( lapack_const(opts.uplo), lapack_const(opts.transA), lapack_const(opts.diag), 
				    &N,
				    h_A, &lda,
				    h_x, &ione );
		    cpu_time = magma_wtime() - cpu_time;
		    cpu_perf = gflops / cpu_time;

		    /* =====================================================================
		       Check the result
		       =================================================================== */
		    // ||b - Ax|| / (||A||*||x||)
		    memcpy(h_x1, h_xmic, N*sizeof(magmaFloatComplex));
		    blasf77_ctrmv( lapack_const(opts.uplo), lapack_const(opts.transA), lapack_const(opts.diag), 
				    &N,
				    h_A, &lda,
				    h_xmic, &ione );

		    blasf77_caxpy( &N, &c_neg_one, h_b1, &ione, h_xmic, &ione );
		    float norm1 =  lapackf77_clange( "M", &N, &ione, h_xmic, &N, work );
		    float normx =  lapackf77_clange( "M", &N, &ione, h_x1, &ione, work );
		    float normA =  lapackf77_clange( "M", &Ak, &Ak, h_A, &lda, work );


		    mic_error = norm1/(normx*normA);

		    printf("%5d   %7.2f (%7.2f)    %7.2f (%7.2f) %8.2e\n",
				    (int) N,
				    mic_perf, 1000.*mic_time,
				    cpu_perf,    1000.*cpu_time,
				    mic_error );

	    } else {
		    printf("%5d   %7.2f (%7.2f)    ---   (  ---  )    ---\n",
				    (int) N,
				    mic_perf, 1000.*mic_time);
	    }

	    TESTING_FREE_HOST( h_A  );
	    TESTING_FREE_HOST( LU  );
	    TESTING_FREE_HOST( LUT );
	    TESTING_FREE_HOST( h_x  );
	    TESTING_FREE_HOST( h_xmic );
	    TESTING_FREE_HOST( h_x1 );

	    TESTING_FREE_DEV( d_A );
	    TESTING_FREE_DEV( d_x );
	}
	if ( opts.niter > 1 ) {
		printf( "\n" );
	}
    }

    magma_queue_destroy( queue );
    magma_finalize();

    return 0;
}
